@@ -1,26 +1,20 @@
 #copyright ReportLab Europe Limited. 2000-2012
 #see license.txt for license details
-__version__=''' $Id: pdfencrypt.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__=''' $Id$ '''
 
 """helpers for pdf encryption/decryption"""
-
-import string, sys, os
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
-
-from reportlab.lib.utils import getStringIO
-import tempfile
-
+import sys, os, tempfile
+from reportlab.lib.utils import getBytesIO, md5, asBytes, int2Byte, char2int, rawUnicode, rawBytes, isPy3
 from reportlab.pdfgen.canvas import Canvas
 from reportlab.pdfbase import pdfutils
+from reportlab.pdfbase.pdfdoc import PDFObject
 from reportlab.platypus.flowables import Flowable
+from reportlab import rl_config
 
 #AR debug hooks - leaving in for now
 CLOBBERID = 0  # set a constant Doc ID to allow comparison with other software like iText
 CLOBBERPERMISSIONS = 0
-DEBUG = 0  # print stuff to trace calculations
+DEBUG = rl_config.debug # print stuff to trace calculations
 
 # permission bits
 reserved1 = 1               # bit 1 must be 0
@@ -34,7 +28,6 @@ higherbits = 0
 for i in range(6,31):
     higherbits = higherbits | (1<<i)
 
-
 # no encryption
 class StandardEncryption:
     prepared = 0
@@ -77,15 +70,15 @@ class StandardEncryption:
     def encode(self, t):
         "encode a string, stream, text"
         if not self.prepared:
-            raise ValueError, "encryption not prepared!"
+            raise ValueError("encryption not prepared!")
         if self.objnum is None:
-            raise ValueError, "not registered in PDF object"
+            raise ValueError("not registered in PDF object")
         return encodePDF(self.key, self.objnum, self.version, t, revision=self.revision)
     def prepare(self, document, overrideID=None):
         # get ready to do encryption
-        if DEBUG: print 'StandardEncryption.prepare(...) - revision %d' % self.revision
+        if DEBUG: print('StandardEncryption.prepare(...) - revision %d' % self.revision)
         if self.prepared:
-            raise ValueError, "encryption already prepared!"
+            raise ValueError("encryption already prepared!")
         # get the unescaped string value of the document id (first array element).
         # we allow one to be passed in instead to permit reproducible tests
         # of our algorithm, but in real life overrideID will always be None
@@ -99,41 +92,40 @@ class StandardEncryption:
                 internalID = "xxxxxxxxxxxxxxxx"
 
         if DEBUG:
-            print 'userPassword    = %s' % self.userPassword
-            print 'ownerPassword   = %s' % self.ownerPassword
-            print 'internalID      = %s' % internalID
-        self.P = int(self.permissionBits() - 2**31L)
+            print('userPassword    = %s' % self.userPassword)
+            print('ownerPassword   = %s' % self.ownerPassword)
+            print('internalID      = %s' % internalID)
+        self.P = int(self.permissionBits() - 2**31)
         if CLOBBERPERMISSIONS: self.P = -44 # AR hack
         if DEBUG:
-            print "self.P          = %s" % repr(self.P)
+            print("self.P          = %s" % repr(self.P))
         self.O = computeO(self.userPassword, self.ownerPassword, self.revision)
         if DEBUG:
-            print "self.O (as hex) = %s" % hexText(self.O)
+            print("self.O (as hex) = %s" % hexText(self.O))
 
         #print "\nself.O", self.O, repr(self.O)
         self.key = encryptionkey(self.userPassword, self.O, self.P, internalID, revision=self.revision)
         if DEBUG:
-            print "self.key (hex)  = %s" % hexText(self.key)
+            print("self.key (hex)  = %s" % hexText(self.key))
         self.U = computeU(self.key, revision=self.revision, documentId=internalID)
         if DEBUG:
-            print "self.U (as hex) = %s" % hexText(self.U)
+            print("self.U (as hex) = %s" % hexText(self.U))
         self.objnum = self.version = None
         self.prepared = 1
     def register(self, objnum, version):
         # enter a new direct object
         if not self.prepared:
-            raise ValueError, "encryption not prepared!"
+            raise ValueError("encryption not prepared!")
         self.objnum = objnum
         self.version = version
     def info(self):
         # the representation of self in file if any (should be None or PDFDict)
         if not self.prepared:
-            raise ValueError, "encryption not prepared!"
+            raise ValueError("encryption not prepared!")
         return StandardEncryptionDictionary(O=self.O, U=self.U, P=self.P, revision=self.revision)
 
-class StandardEncryptionDictionary:
+class StandardEncryptionDictionary(PDFObject):
     __RefOnly__ = 1
-    __PDFObject__ = True
     def __init__(self, O, U, P, revision):
         self.O, self.U, self.P = O,U,P
         self.revision = revision
@@ -160,64 +152,54 @@ padding = """
 28 BF 4E 5E 4E 75 8A 41 64 00 4E 56 FF FA 01 08
 2E 2E 00 B6 D0 68 3E 80 2F 0C A9 FE 64 53 69 7A
 """
-if hasattr(padding,'join'):
+if isPy3:
+    def xorKey(num,key):
+        "xor's each byte of the key with the number, which is <256"
+        if num==0: return key
+        return bytes(num^k for k in key)
+else:
     def xorKey(num,key):
         "xor's each bytes of the key with the number, which is <256"
         if num==0: return key
-        from operator import xor
-        return ''.join(map(chr,map(xor,len(key)*[num],map(ord,key))))
-else:
-    def xorKey(num, key):
-        "xor's each bytes of the key with the number, which is <256"
-        from operator import xor
-        out = ''
-        for ch in key:
-            out = out + chr(xor(num, ord(ch)))
-        return out
-
-def hexchar(x):
-    return chr(string.atoi(x, 16))
+        return ''.join(chr(num^ord(k)) for k in key)
 
 def hexText(text):
     "a legitimate way to show strings in PDF"
-    out = ''
-    for char in text:
-        out = out + '%02X' % ord(char)
-    return '<' + out + '>'
+    return '<' + ''.join('%02X' % ord(c) for c in rawUnicode(text)) + '>'
 
 def unHexText(hexText):
-    assert hexText[0] == '<', 'bad hex text'
-    assert hexText[-1] == '>', 'bad hex text'
+    equalityCheck(hexText[0], '<', 'bad hex text')
+    equalityCheck(hexText[-1], '>', 'bad hex text')
     hexText = hexText[1:-1]
-    out = ''
+    out = b''
     for i in range(int(len(hexText)/2.0)):
         slice = hexText[i*2: i*2+2]
-        char = chr(eval('0x'+slice))
+        char = int2Byte(eval('0x'+slice))
         out = out + char
     return out
 
-PadString = string.join(map(hexchar, string.split(string.strip(padding))), "")
+PadString = rawBytes(''.join(chr(int(c, 16)) for c in padding.strip().split()))
 
 def encryptionkey(password, OwnerKey, Permissions, FileId1, revision=2):
     # FileId1 is first string of the fileid array
     # add padding string
     #AR force same as iText example
     #Permissions =  -1836   #int(Permissions - 2**31)
-    password = password + PadString
+    password = asBytes(password) + PadString
     # truncate to 32 bytes
     password = password[:32]
     # translate permissions to string, low order byte first
     p = Permissions# + 2**32L
-    permissionsString = ""
+    permissionsString = b""
     for i in range(4):
         byte = (p & 0xff)    # seems to match what iText does
         p = p>>8
-        permissionsString = permissionsString + chr(byte % 256)
+        permissionsString += int2Byte(byte % 256)
 
-    hash = md5(password)
-    hash.update(OwnerKey)
-    hash.update(permissionsString)
-    hash.update(FileId1)
+    hash = md5(asBytes(password))
+    hash.update(asBytes(OwnerKey))
+    hash.update(asBytes(permissionsString))
+    hash.update(asBytes(FileId1))
 
     md5output = hash.digest()
 
@@ -227,23 +209,24 @@ def encryptionkey(password, OwnerKey, Permissions, FileId1, revision=2):
         for x in range(50):
             md5output = md5(md5output).digest()
         key = md5output[:16]
-    if DEBUG: print 'encryptionkey(%s,%s,%s,%s,%s)==>%s' % tuple(map(lambda x: hexText(str(x)),(password, OwnerKey, Permissions, FileId1, revision, key)))
+    if DEBUG: print('encryptionkey(%s,%s,%s,%s,%s)==>%s' % tuple([hexText(str(x)) for x in (password, OwnerKey, Permissions, FileId1, revision, key)]))
     return key
 
 def computeO(userPassword, ownerPassword, revision):
     from reportlab.lib.arciv import ArcIV
     #print 'digest of hello is %s' % md5('hello').digest()
     assert revision in (2,3), 'Unknown algorithm revision %s' % revision
-    if ownerPassword in (None, ''):
+    if not ownerPassword:
         ownerPassword = userPassword
 
-    ownerPad = ownerPassword + PadString
+    ownerPad = asBytes(ownerPassword) + PadString
     ownerPad = ownerPad[0:32]
 
-    password = userPassword + PadString
+    password = asBytes(userPassword) + PadString
     userPad = password[:32]
 
     digest = md5(ownerPad).digest()
+    if DEBUG: print('PadString=%s\nownerPad=%s\npassword=%s\nuserPad=%s\ndigest=%s\nrevision=%s' % (ascii(PadString),ascii(ownerPad),ascii(password),ascii(userPad),ascii(digest),revision))
     if revision == 2:
         O = ArcIV(digest[:5]).encode(userPad)
     elif revision == 3:
@@ -254,7 +237,7 @@ def computeO(userPassword, ownerPassword, revision):
         for i in range(20):
             thisKey = xorKey(i, digest)
             O = ArcIV(thisKey).encode(O)
-    if DEBUG: print 'computeO(%s,%s,%s)==>%s' % tuple(map(lambda x: hexText(str(x)),(userPassword, ownerPassword, revision,O)))
+    if DEBUG: print('computeO(%s,%s,%s)==>%s' % tuple([hexText(str(x)) for x in (userPassword, ownerPassword, revision,O)]))
     return O
 
 def computeU(encryptionkey, encodestring=PadString,revision=2,documentId=None):
@@ -264,16 +247,16 @@ def computeU(encryptionkey, encodestring=PadString,revision=2,documentId=None):
     elif revision == 3:
         assert documentId is not None, "Revision 3 algorithm needs the document ID!"
         h = md5(PadString)
-        h.update(documentId)
+        h.update(rawBytes(documentId))
         tmp = h.digest()
         tmp = ArcIV(encryptionkey).encode(tmp)
         for n in range(1,20):
             thisKey = xorKey(n, encryptionkey)
             tmp = ArcIV(thisKey).encode(tmp)
         while len(tmp) < 32:
-            tmp = tmp + '\000'
+            tmp += b'\0'
         result = tmp
-    if DEBUG: print 'computeU(%s,%s,%s,%s)==>%s' % tuple(map(lambda x: hexText(str(x)),(encryptionkey, encodestring,revision,documentId,result)))
+    if DEBUG: print('computeU(%s,%s,%s,%s)==>%s' % tuple([hexText(str(x)) for x in (encryptionkey, encodestring,revision,documentId,result)]))
     return result
 
 def checkU(encryptionkey, U):
@@ -281,8 +264,8 @@ def checkU(encryptionkey, U):
     #print len(decoded), len(U), len(PadString)
     if decoded!=PadString:
         if len(decoded)!=len(PadString):
-            raise ValueError, "lengths don't match! (password failed)"
-        raise ValueError, "decode of U doesn't match fixed padstring (password failed)"
+            raise ValueError("lengths don't match! (password failed)")
+        raise ValueError("decode of U doesn't match fixed padstring (password failed)")
 
 def encodePDF(key, objectNumber, generationNumber, string, revision=2):
     "Encodes a string or stream"
@@ -291,12 +274,12 @@ def encodePDF(key, objectNumber, generationNumber, string, revision=2):
     newkey = key
     n = objectNumber
     for i in range(3):
-        newkey = newkey + chr(n & 0xff)
+        newkey += int2Byte(n & 0xff)
         n = n>>8
     # extend 2 bytes of the generationNumber
     n = generationNumber
     for i in range(2):
-        newkey = newkey + chr(n & 0xff)
+        newkey += int2Byte(n & 0xff)
         n = n>>8
     md5output = md5(newkey).digest()
     if revision == 2:
@@ -306,39 +289,40 @@ def encodePDF(key, objectNumber, generationNumber, string, revision=2):
     from reportlab.lib.arciv import ArcIV
     encrypted = ArcIV(key).encode(string)
     #print 'encrypted=', hexText(encrypted)
-    if DEBUG: print 'encodePDF(%s,%s,%s,%s,%s)==>%s' % tuple(map(lambda x: hexText(str(x)),(key, objectNumber, generationNumber, string, revision,encrypted)))
+    if DEBUG: print('encodePDF(%s,%s,%s,%s,%s)==>%s' % tuple([hexText(str(x)) for x in (key, objectNumber, generationNumber, string, revision,encrypted)]))
     return encrypted
 
-    ######################################################################
-    #
-    #  quick tests of algorithm, should be moved elsewhere
-    #
-    ######################################################################
-
+def equalityCheck(observed,expected,label):
+    assert observed==expected,'%s\n expected=%s\n observed=%s' % (label,expected,observed)
+######################################################################
+#
+#  quick tests of algorithm, should be moved elsewhere
+#
+######################################################################
 def test():
     # do a 40 bit example known to work in Acrobat Reader 4.0
-    enc = StandardEncryption('userpass','ownerpass', strength=40)
-    enc.prepare(None, overrideID = 'xxxxxxxxxxxxxxxx')
+    enc = StandardEncryption('User','Owner', strength=40)
+    enc.prepare(None, overrideID='xxxxxxxxxxxxxxxx')
 
-    expectedO = '<6A835A92E99DCEA39D51CF34FDBDA42162690D2BD5F8E08E3008F91FE5B8512E>'
-    expectedU = '<9997BDB61E7F288DAE6A8C4246A8F9CDCDBBC3D909D703CABA5D65A0CC6D4083>'
-    expectedKey = '<A3A68B5CB1>'  # 5 byte key = 40 bits
+    expectedO = '<FA7F558FACF8205D25A7F1ABFA02629F707AE7B0211A2BB26F5DF4C30F684301>'
+    expectedU = '<09F26CF46190AF8F93B304AD50C16B615DC43C228C9B2D2EA34951A80617B2B1>'
+    expectedKey = '<BB2C00EB3D>'    # 5 byte key = 40 bits
 
-    assert hexText(enc.O) == expectedO, '40 bit unexpected O value %s' % hexText(enc.O)
-    assert hexText(enc.U) == expectedU, '40 bit unexpected U value %s' % hexText(enc.U)
-    assert hexText(enc.key) == expectedKey, '40 bit unexpected key value %s' % hexText(enc.key)
+    equalityCheck(hexText(enc.O),expectedO, '40 bit O value')
+    equalityCheck(hexText(enc.U),expectedU, '40 bit U value')
+    equalityCheck(hexText(enc.key),expectedKey, '40 bit key value')
 
     # now for 128 bit example
     enc = StandardEncryption('userpass','ownerpass', strength=128)
     enc.prepare(None, overrideID = 'xxxxxxxxxxxxxxxx')
 
-    expectedO = '<19BDBD240E0866B84C49AEEF7E2350045DB8BDAE96E039BF4E3F12DAC3427DB6>'
-    expectedU = '<564747DADFF35F5F2078A2CA1705B50800000000000000000000000000000000>'
-    expectedKey = '<DC1E019846B1EEABA0CDB8ED6D53B5C4>'  # 16 byte key = 128 bits
+    expectedO = '<68E5704AC779A5F0CD89704406587A52F25BF61CADC56A0F8DB6C4DB0052534D>'
+    expectedU = '<A9AE45CDE827FE0B7D6536267948836A00000000000000000000000000000000>'
+    expectedKey = '<13DDE7585D9BE366C976DDD56AF541D1>'  # 16 byte key = 128 bits
 
-    assert hexText(enc.O) == expectedO, '128 bit unexpected O value %s' % hexText(enc.O)
-    assert hexText(enc.U) == expectedU, '128 bit unexpected U value %s' % hexText(enc.U)
-    assert hexText(enc.key) == expectedKey, '128 bit unexpected key value %s' % hexText(enc.key)
+    equalityCheck(hexText(enc.O), expectedO, '128 bit O value')
+    equalityCheck(hexText(enc.U), expectedU, '128 bit U value')
+    equalityCheck(hexText(enc.key), expectedKey, '128 key value')
 
     ######################################################################
     #
@@ -403,12 +387,12 @@ def encryptPdfInMemory(inputPDF,
 See http://developer.reportlab.com''')
 
     (bboxInfo, pickledForms) = storeFormsInMemory(inputPDF, all=1, BBoxes=1)
-    names = bboxInfo.keys()
+    names = list(bboxInfo.keys())
 
     firstPageSize = bboxInfo['PageForms0'][2:]
 
     #now make a new PDF document
-    buf = getStringIO()
+    buf = getBytesIO()
     canv = Canvas(buf, pagesize=firstPageSize)
 
     # set a standard ID while debugging
@@ -503,7 +487,7 @@ See PdfEncryptIntro.pdf for more information.
     argv = list(sys_argv)[1:]
     if len(argv)>0:
         if argv[0] == '-h' or argv[0] == 'help':
-            print usage
+            print(usage)
             return
         if len(argv)<2:
             raise ValueError("Must include a filename and one or more arguments!")
@@ -563,11 +547,8 @@ See PdfEncryptIntro.pdf for more information.
             if thisarg[0] in argv:
                 pos = argv.index(thisarg[0])
                 if thisarg[0] in binaryrequired:
-                    #try:
                     if argv[pos+1] not in ('1', '0'):
-                        raise "%s value must be either '1' or '0'!" % thisarg[1]
-                    #except:
-                        #raise "Unable to set %s." % thisarg[4]
+                        raise ValueError("%s value must be either '1' or '0'!" % thisarg[1])
                 try:
                     if argv[pos+1] not in known_modes:
                         if thisarg[0] in binaryrequired:
@@ -575,7 +556,7 @@ See PdfEncryptIntro.pdf for more information.
                         else:
                             exec(thisarg[1] +' = argv[pos+1]')
                         if verbose:
-                            print "%s set to: '%s'." % (thisarg[3], argv[pos+1])
+                            print("%s set to: '%s'." % (thisarg[3], argv[pos+1]))
                         argv.remove(argv[pos+1])
                         argv.remove(thisarg[0])
                 except:
@@ -583,17 +564,17 @@ See PdfEncryptIntro.pdf for more information.
 
         if verbose>4:
             #useful if feeling paranoid and need to double check things at this point...
-            print "\ninfile:", infile
-            print "STRENGTH:", STRENGTH
-            print "SAVEFILE:", SAVEFILE
-            print "USER:", USER
-            print "OWNER:", OWNER
-            print "PRINTABLE:", PRINTABLE
-            print "MODIFIABLE:", MODIFIABLE
-            print "COPYPASTABLE:", COPYPASTABLE
-            print "ANNOTATABLE:", ANNOTATABLE
-            print "SAVEFILE:", SAVEFILE
-            print "VERBOSE:", verbose
+            print("\ninfile:", infile)
+            print("STRENGTH:", STRENGTH)
+            print("SAVEFILE:", SAVEFILE)
+            print("USER:", USER)
+            print("OWNER:", OWNER)
+            print("PRINTABLE:", PRINTABLE)
+            print("MODIFIABLE:", MODIFIABLE)
+            print("COPYPASTABLE:", COPYPASTABLE)
+            print("ANNOTATABLE:", ANNOTATABLE)
+            print("SAVEFILE:", SAVEFILE)
+            print("VERBOSE:", verbose)
 
 
         if SAVEFILE == 'encrypted.pdf':
@@ -608,21 +589,21 @@ See PdfEncryptIntro.pdf for more information.
                                     strength=STRENGTH)
 
         if verbose:
-            print "wrote output file '%s'(%s bytes)\n  owner password is '%s'\n  user password is '%s'" % (SAVEFILE, filesize, OWNER, USER)
+            print("wrote output file '%s'(%s bytes)\n  owner password is '%s'\n  user password is '%s'" % (SAVEFILE, filesize, OWNER, USER))
 
         if len(argv)>0:
-            raise "\nUnrecognised arguments : %s\nknown arguments are:\n%s" % (str(argv)[1:-1], known_modes)
+            raise valueError("\nUnrecognised arguments : %s\nknown arguments are:\n%s" % (str(argv)[1:-1], known_modes))
     else:
-        print usage
+        print(usage)
 
 def main():
     from reportlab.rl_config import verbose
     scriptInterp()
 
 if __name__=="__main__": #NO RUNTESTS
-    a = filter(lambda x: x[:7]=='--debug',sys.argv)
+    a = [x for x in sys.argv if x[:7]=='--debug']
     if a:
-        sys.argv = filter(lambda x: x[:7]!='--debug',sys.argv)
+        sys.argv = [x for x in sys.argv if x[:7]!='--debug']
         DEBUG = len(a)
     if '--test' in sys.argv: test()
     else: main()
