@@ -1,6 +1,6 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
-__version__=''' $Id: test_platypus_general.py 3660 2010-02-08 18:17:33Z damian $ '''
+__version__=''' $Id$ '''
 
 #tests and documents Page Layout API
 __doc__="""This is not obvious so here's a brief explanation.  This module is both
@@ -13,7 +13,7 @@ and drawn into.
 """
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
 setOutDir(__name__)
-import string, copy, sys, os
+import copy, sys, os
 from reportlab.pdfgen import canvas
 from reportlab import platypus
 from reportlab.platypus import BaseDocTemplate, PageTemplate, Flowable, FrameBreak
@@ -22,7 +22,7 @@ from reportlab.lib.units import inch, cm
 from reportlab.lib.styles import PropertySet, getSampleStyleSheet, ParagraphStyle
 from reportlab.lib import colors
 from reportlab.rl_config import defaultPageSize
-from reportlab.lib.utils import haveImages, _RL_DIR, rl_isfile, open_for_read, fileName2Utf8
+from reportlab.lib.utils import haveImages, _RL_DIR, rl_isfile, open_for_read, fileName2FSEnc
 import unittest
 from reportlab.lib.testutils import testsFolder
 if haveImages:
@@ -30,7 +30,7 @@ if haveImages:
     if not rl_isfile(_GIF): _GIF = None
 else:
     _GIF = None
-if _GIF: _GIFUTF8=fileName2Utf8(_GIF)
+if _GIF: _GIFFSEnc=fileName2FSEnc(_GIF)
 
 _JPG = os.path.join(testsFolder,'..','docs','images','lj8100.jpg')
 if not rl_isfile(_JPG): _JPG = None
@@ -71,20 +71,20 @@ def getParagraphs(textBlock):
     """Within the script, it is useful to whack out a page in triple
     quotes containing separate paragraphs. This breaks one into its
     constituent paragraphs, using blank lines as the delimiter."""
-    lines = string.split(textBlock, '\n')
+    lines = textBlock.split('\n')
     paras = []
     currentPara = []
     for line in lines:
-        if len(string.strip(line)) == 0:
+        if len(line.strip()) == 0:
             #blank, add it
             if currentPara != []:
-                paras.append(string.join(currentPara, '\n'))
+                paras.append('\n'.join(currentPara))
                 currentPara = []
         else:
             currentPara.append(line)
     #...and the last one
     if currentPara != []:
-        paras.append(string.join(currentPara, '\n'))
+        paras.append('\n'.join(currentPara))
 
     return paras
 
@@ -312,11 +312,11 @@ def getCommentary():
     story.append(FrameBreak())
     if _GIF:
         story.append(Paragraph("""We can use images via the file name""", styleSheet['BodyText']))
-        code('''    story.append(platypus.Image('%s'))'''%_GIFUTF8)
-        code('''    story.append(platypus.Image(fileName2Utf8('%s')))''' % _GIFUTF8)
+        code('''    story.append(platypus.Image('%s'))'''%_GIFFSEnc)
+        code('''    story.append(platypus.Image(fileName2FSEnc('%s')))''' % _GIFFSEnc)
         story.append(Paragraph("""They can also be used with a file URI or from an open python file!""", styleSheet['BodyText']))
-        code('''    story.append(platypus.Image('%s'))'''% getFurl(_GIFUTF8))
-        code('''    story.append(platypus.Image(open_for_read('%s','b')))''' % _GIFUTF8)
+        code('''    story.append(platypus.Image('%s'))'''% getFurl(_GIFFSEnc))
+        code('''    story.append(platypus.Image(open_for_read('%s','b')))''' % _GIFFSEnc)
         story.append(FrameBreak())
         story.append(Paragraph("""Images can even be obtained from the internet.""", styleSheet['BodyText']))
         code('''    img = platypus.Image('http://www.reportlab.com/rsrc/encryption.gif')
@@ -358,7 +358,7 @@ def getExamples():
 
     story.append(Paragraph("""Same but with justification 1.5 extra leading and green text.""", styleSheet['BodyText']))
     p = Paragraph("""
-        <para align=justify leading=+1.5 fg=green><font color=red>Platypus</font> is all about fitting objects into frames on the page.  You
+        <para align=justify leading="+1.5" fg=green><font color=red>Platypus</font> is all about fitting objects into frames on the page.  You
         are looking at a fairly simple Platypus paragraph in Debug mode.
         It has some gridlines drawn around it to show the left and right indents,
         and the space before and after, all of which are attributes set in
@@ -496,7 +496,7 @@ def getExamples():
         story.append(Paragraph("Here is an Image flowable obtained from a string filename.",styleSheet['Italic']))
         story.append(platypus.Image(_GIF))
         story.append(Paragraph( "Here is an Image flowable obtained from a utf8 filename.", styleSheet['Italic']))
-        #story.append(platypus.Image(fileName2Utf8(_GIF)))
+        #story.append(platypus.Image(fileName2FSEnc(_GIF)))
         story.append(Paragraph("Here is an Image flowable obtained from a string file url.",styleSheet['Italic']))
         story.append(platypus.Image(getFurl(_GIF)))
         story.append(Paragraph("Here is an Image flowable obtained from an open file.",styleSheet['Italic']))
@@ -537,8 +537,8 @@ class AndyTemplate(BaseDocTemplate):
             self.handle_flowable(flowables)
 
     def build(self, flowables1, flowables2):
-        assert filter(lambda x: not isinstance(x,Flowable), flowables1)==[], "flowables1 argument error"
-        assert filter(lambda x: not isinstance(x,Flowable), flowables2)==[], "flowables2 argument error"
+        assert [x for x in flowables1 if not isinstance(x,Flowable)]==[], "flowables1 argument error"
+        assert [x for x in flowables2 if not isinstance(x,Flowable)]==[], "flowables2 argument error"
         self._startBuild()
         while (len(flowables1) > 0 or len(flowables1) > 0):
             self.clean_hanging()
@@ -548,7 +548,7 @@ class AndyTemplate(BaseDocTemplate):
         self._endBuild()
 
 def showProgress(pageNo):
-    print 'CALLBACK SAYS: page %d' % pageNo
+    print('CALLBACK SAYS: page %d' % pageNo)
 
 
 def run():
